#include <vdw.h>
#include <ntddk.h>
#include "NTPCFOMDriver.h"
#include "NTPCFOMDevice.h"
#include "..\NTPCFOMioctl.h"
#include "NTPCFOMHelper.h"

#pragma code_seg()

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Kernel Mode delay function
 *
 * Performs a required delay. The usage of KeStallExecutionProcessor is
 *	very nasty, but it happends only if SysDelay is called in the context of
 *	our DPC routine (which is only called if a card change was detected).
 *
 *	For 'normal' IO we have Irql < DISPATCH_LEVEL, so if the reader is polled
 *	while waiting for response we will not block the entire system
 *
 * Arguments:
 *	Timeout		delay in milli seconds
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::SysDelay(ULONG dw_Timeout)
{
	LARGE_INTEGER	SysTimeout;


	if( KeGetCurrentIrql() >= DISPATCH_LEVEL )
	{
		ULONG	Cnt = 20 * dw_Timeout;

		while( Cnt-- )
		{
			//	KeStallExecutionProcessor: counted in us
			KeStallExecutionProcessor( 50 );
		}
	}
	else
	{
		SysTimeout.QuadPart = (LONGLONG)-10 * 1000 * dw_Timeout;

		//	KeDelayExecutionThread: counted in 100 ns
		KeDelayExecutionThread( KernelMode, FALSE, &SysTimeout );
	}

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Carica il firmware nella RAM della scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS NTPCFOMDevice::LoadDeviceFirmware()
{
	FILE_STANDARD_INFORMATION StandardInfo;
	NTSTATUS	status;
	UCHAR*		pby_fileBuffer;
	UCHAR*		pby_memoryBuffer;
	ULONG		dw_fileSize;
	ULONG		dw_fileRead;
	UCHAR*		pby_tmp;
	UCHAR		by_tmp;
	USHORT		w_tmp;

	KFile		m_file;

	// The name of the file to open is passed in the input buffer
	KUstring name( m_FirmwareImage );

	status = m_file.OpenCreate(
				name,
				NULL,
				FILE_GENERIC_READ | SYNCHRONIZE,
				OBJ_CASE_INSENSITIVE,
				0,
				FILE_SHARE_READ,
				FILE_OPEN,
				FILE_NON_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT);

	if(!NT_SUCCESS(status))
	{
#if DBG
		t << "Failed to create open device firmware file " << name << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}

	m_file.Query(&StandardInfo);

	if(StandardInfo.EndOfFile.LowPart >= 0xfb80)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Wrong firmware file length " << StandardInfo.EndOfFile.LowPart << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			

	}

#if DBG
	t << "File size is " << StandardInfo.EndOfFile.LowPart << EOL;
#endif
	dw_fileSize	=	StandardInfo.EndOfFile.LowPart;

	//	Lancio il programma predefinito caricato nella scheda.
	//	Questo programma e' un loader che consente di caricare qualunque altro firmware
	//	compatibile con la scheda
	
	PC_PCCIO_MODE2_SET;
	PC_PCCIO_RESET;

	//	Delay 1.2 secondi

	SysDelay(1200);
	
	pby_fileBuffer	=	new	UCHAR[dw_fileSize];
	
	if(pby_fileBuffer==NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Error allocating file firmware buffer!" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}

	RtlZeroMemory(pby_fileBuffer, dw_fileSize);

	m_file.Read(pby_fileBuffer, dw_fileSize, &dw_fileRead, 0);
	
	//	Scarico il programma nella scheda

	DeviceIoMemWrite((USHORT)0, (USHORT)(dw_fileSize&0xFFFF), pby_fileBuffer);

	//	Ricarco il programma dalla scheda

	pby_memoryBuffer	=	new	UCHAR[dw_fileSize];

	if(pby_memoryBuffer==NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Error allocating memory firmware buffer!" << EOL;
#endif
		delete[]	pby_fileBuffer;

		return STATUS_UNSUCCESSFUL;			
	}

	RtlZeroMemory(pby_memoryBuffer, dw_fileSize);

	status	=	DeviceIoMemRead(pby_memoryBuffer, (USHORT)0, (USHORT)(dw_fileSize&0xFFFF));

	if(!NT_SUCCESS(status))
	{
#if DBG
		t << "Error reading program PCFOM memory!" << EOL;
#endif
		delete[]	pby_fileBuffer;
		delete[]	pby_memoryBuffer;
		
		return	status;
	}

	//	Confronto quanto riletto con il contenuto del file

	if(dw_fileSize != RtlCompareMemory(pby_fileBuffer, pby_memoryBuffer, dw_fileSize))
	{
#if DBG
		t << "Error comparing memory with buffer!" << EOL;
#endif
		status	=	STATUS_UNSUCCESSFUL;
	}
	else
	{
#if DBG
		t << "Memory and file contain the same image!" << EOL;
#endif
	}

	delete[]	pby_fileBuffer;
	delete[]	pby_memoryBuffer;

	if(status	==	STATUS_UNSUCCESSFUL)
		return	status;

	PC_PCCIO_MODE1_SET;
	PC_PCCIO_RESET;

	//	Delay 1 secondo

	SysDelay(1200);

	status	=	DeviceIsFirmwareRunning();

	if(NT_SUCCESS(status))
	{
#if DBG
		t << "Firmware appear to be loaded and running correctly!" << EOL;
#endif
		status	=	GetConfig();
	
		if(NT_SUCCESS(status))
			status	=	ResetTracesTimer();
	}
	else
	{
#if DBG
		t << "Firmware appear to NOT be loaded and running correctly!" << EOL;
#endif
	}

	return	status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Resetta la scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::Reset()
{
	PC_PCCIO_MODE2_SET;
	PC_PCCIO_RESET;

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Riavvia il programma della scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::Restart()
{
	LoadDeviceFirmware();

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Verifica che il firmware sia caricato e funzioni regolarmente
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::DeviceIsFirmwareRunning()
{
#if 0
	
	UCHAR	by_dato;

	by_dato=DeviceIoRead();

#if DBG
	t << "by_dato=" << by_dato << EOL;
#endif	
	if(by_dato	!=	0xaa)
		return STATUS_UNSUCCESSFUL;

	by_dato=DeviceIoRead();

#if DBG
	t << "by_dato=" << by_dato << EOL;
#endif

	if(by_dato	!=	0x55)
		return STATUS_UNSUCCESSFUL;
#else

	if(DeviceIoRead()	!=	0xaa)
		return STATUS_UNSUCCESSFUL;

	if(DeviceIoRead()	!=	0x55)
		return STATUS_UNSUCCESSFUL;

#endif
	
	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Funzione di lettura / scrittura sa e per la scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::DeviceIoMemWrite(USHORT w_Address, USHORT w_Size, UCHAR *pby_Data)
{
	NTSTATUS status;

	if(pby_Data==NULL || w_Size == 0)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Data == NULL or size == 0" << EOL;
#endif

		return STATUS_UNSUCCESSFUL;			
	}
	
	PCCIO_FR_STR*	p_frame;

	p_frame	=	new	PCCIO_FR_STR(PCCIO_CMD_DIRECT_MEM_WR, w_Address, w_Size, pby_Data);

	if(p_frame == NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "p_frame == NULL" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}
	
	status	=	DeviceTxFrame(p_frame);

	if(NT_SUCCESS(status))
	{
		if(p_frame->by_cmd	> PCCIO_CMD_DIRECT_END && p_frame->by_cmd <= PCCIO_CMD_END)
		{
			status	=	DecodeRx(p_frame);
		}
	}

	delete	p_frame;

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Funzione per la lettura della memoria dalla scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS NTPCFOMDevice::DeviceIoMemRead(UCHAR *pby_Data, USHORT w_Address, USHORT w_Size)
{
	NTSTATUS status;

	if(pby_Data==NULL || w_Size == 0)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Data == NULL or size == 0" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}
	
	PCCIO_FR_STR*	p_frame;

	p_frame	=	new	PCCIO_FR_STR(PCCIO_CMD_DIRECT_MEM_RD, w_Address, w_Size, pby_Data);

	if(p_frame == NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "p_frame == NULL" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}
	
	status	=	DeviceTxFrame(p_frame);
	
	if(NT_SUCCESS(status))
	{
		if(p_frame->by_cmd	> PCCIO_CMD_DIRECT_END && p_frame->by_cmd <= PCCIO_CMD_END)
		{
			status	=	DecodeRx(p_frame);
		}
	}

	delete	p_frame;

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Decodifica i frame ricevuti dalla scheda PCFOM
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::DecodeRx(PCCIO_FR_STR*  p_Frame)
{
	NTSTATUS status;
	PCCIO_FR_STR* p_tmpFrame;   // Puntatore a un frame temporaneo.
	PUCHAR   pby_tmpFrame;   // Puntatore a un frame temporaneo.

// In funzione del comando contenuto nel frame.
	switch(p_Frame->by_cmd)
	{
		case PCCIO_CMD_LOOP_BACK_TST:

		// Trasmetto il frame ricevuto.
			status	= DeviceTxFrame(p_Frame);

		// Distruggo il frame ricevuto.
			delete	p_Frame;

			break;

		// Alloca un buffer e ne rende l'indirizzo.
		case PCCIO_CMD_MALLOC:

		// Alloco il buffer grande quanto specificato nell'argomento e
		// Imposto l'argomento con l'indirizzo del buffer allocato.
			pby_tmpFrame	= new	UCHAR[(p_Frame->w_arg)];
			p_Frame->w_arg = (USHORT)pby_tmpFrame;

		// Trasmetto il frame ricevuto.
			status	= DeviceTxFrame(p_Frame);

		// Se non sono riuscito a trasmettere l'indirizzo del frame
		// distruggo il buffer allocato.
			if(!NT_SUCCESS(status))
				delete	pby_tmpFrame;

		// Distruggo il frame ricevuto.
			delete	p_Frame;

			break;

		// Libera un buffer allocato.
		case PCCIO_CMD_FREE:

		// Libero il buffer allocato puntato dall'argomento.
			//delete free((void*)((*pp_frame)->w_arg));
//				delete

		// Distruggo il frame ricevuto.
			delete	p_Frame;
	}

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Invia sul BUS un BYTE alla scheda PCFOM gestendo il timeout ed errori
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::DeviceIoWrite(UCHAR	by_dato)
{
	ULONG	m_timeoutCt	=	m_IoTimeout;

	do
	{
		if(m_IoTimeout > 0 &&  !m_timeoutCt--)
		{
			//	Se il timeout e' abilitato
			if(m_IoTimeout)
			{
				DeviceIoAbort(PCFOM_IO_ERROR_WRITING);
				return STATUS_UNSUCCESSFUL;			
			}
		}
	}
	while(PC_PCCIO_WR_BUSY_TST);

	PC_PCCIO_PORT_WR(by_dato);

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Invia sul BUS una WORD alla scheda PCFOM gestendo il timeout ed errori
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::DeviceIoWrite(USHORT	w_dato)
{
	if(DeviceIoWrite((UCHAR)(((w_dato)>>8)&0xff))!=STATUS_SUCCESS)
		return STATUS_UNSUCCESSFUL;			

	if(DeviceIoWrite((UCHAR)(w_dato&0xff))!=STATUS_SUCCESS)
		return STATUS_UNSUCCESSFUL;			

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Verifica sul BUS se c'e' un frame disponibile in arrivo dalla PCFOM 
 *		  gestendo timeout e errori
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::IsFrameAvailable()
{
	UCHAR	by_dato;
	UCHAR	by_register;
	ULONG	m_timeoutCt	=	m_IoTimeout;

	do
	{
		if(m_IoTimeout > 0 && !m_timeoutCt--)
		{
			//	Se il timeout e' abilitato
			if(m_IoTimeout)
			{
				DeviceIoAbort(PCFOM_IO_ERROR_WAITING|PCFOM_IO_ERROR_READING);
				return	STATUS_UNSUCCESSFUL;
			}
		}
	}
	while(!PC_PCCIO_RD_BUSY_TST);

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge dal BUS un BYTE gestendo timeout e errori 
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


SHORT	NTPCFOMDevice::DeviceIoRead()
{
	UCHAR	by_dato;
	ULONG	m_timeoutCt	=	m_IoTimeout;
	
	do
	{
		if(m_IoTimeout > 0 &&  !m_timeoutCt--)
		{
			//	Se il timeout e' abilitato
			if(m_IoTimeout)
			{
				DeviceIoAbort(PCFOM_IO_ERROR_READING);
				return (SHORT)-1;			
			}
		}
	}
	while(!PC_PCCIO_RD_BUSY_TST);

	by_dato	=	PC_PCCIO_PORT_RD;

	return by_dato;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge dal BUS una WORD gestendo timeout e errori 
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

LONG	NTPCFOMDevice::DeviceIoReadW()
{
	SHORT	s_dato;
	USHORT	w_dato;

	if((s_dato	=	DeviceIoRead())	==	(SHORT)-1)
		return	(LONG)-1;

	w_dato	=	s_dato&0xff;
	w_dato	<<=	8;

	if((s_dato	=	DeviceIoRead())	==	(SHORT)-1)
		return	(LONG)-1;

	w_dato	|=	s_dato&0xff;

	return	(USHORT)(w_dato&0xffff);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Trasmette un intero frame alla PCFOM con il protocollo PARBUS
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::DeviceTxFrame(PCCIO_FR_STR*   p_frameTx)
{
	USHORT  w_checkSum;
	USHORT  w_rxCheckSum;
	SHORT	w_rxd;

	if(p_frameTx->w_size && p_frameTx->pby_data==NULL)
		return STATUS_UNSUCCESSFUL;

// 0 UCHAR Invio il carattere per il sincronismo di inizio trasmissione.
	if(DeviceIoWrite((UCHAR)0x55)!=STATUS_SUCCESS)
	{
#if DBG
		t << "DeviceTxFrame: Error Sending Sync" << EOL;
#endif
		return	STATUS_UNSUCCESSFUL;
	}

// 1 UCHAR Invio il comando
	if(DeviceIoWrite(p_frameTx->by_cmd)!=STATUS_SUCCESS)
	{
#if DBG
		t << "DeviceTxFrame: Error Sending p_frameTx->by_cmd" << EOL;
#endif
		return	STATUS_UNSUCCESSFUL;
	}

// 2/3 UCHAR Invio l'argomento del comando

	if(DeviceIoWrite(p_frameTx->w_arg)!=STATUS_SUCCESS)
	{
#if DBG
		t << "DeviceTxFrame: Error Sending p_frameTx->w_arg" << EOL;
#endif
		return	STATUS_UNSUCCESSFUL;
	}

// 4/5 UCHAR  Invio la dimensione del frame.

	if(DeviceIoWrite(p_frameTx->w_size)!=STATUS_SUCCESS)
	{
#if DBG
		t << "DeviceTxFrame: Error Sending p_frameTx->w_size" << EOL;
#endif
		return	STATUS_UNSUCCESSFUL;
	}

// Calcolo in checkSum dell'header frame.
// Praticamente il checkSum del frame senza campo data.
// Viene usato per verificare il comando prima che venga eseguito.

	w_checkSum = p_frameTx->HeaderChecksum();

// 6/7 UCHAR  Invio il checkSum dell'header.

	if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
	{
#if DBG
		t << "DeviceTxFrame: Error Sending checksum" << EOL;
#endif
		return	STATUS_UNSUCCESSFUL;
	}

// Se il checkSum dell'header ricevuto e' diverso da quello calcolato
// Esco con errore.

	if(w_checkSum != DeviceIoReadW())
	{
#if DBG
		t << "DeviceTxFrame: Error Comparing checksum 1" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;
	}

// Se il comando e' <= di PCCIO_CMD_DIRECT_END

	if(p_frameTx->by_cmd <= PCCIO_CMD_DIRECT_END)
	{
#if DBG
//		t << "DeviceTxFrame: Calling DeviceIoDirectFrame" << EOL;
#endif
		// Chiamo la macro specializzata per trasmettere i frame direttamente.
		return	DeviceIoDirectFrame(p_frameTx);
	}
// Se c'e' il campo data.
	else if(p_frameTx->w_size)
	{
	 // Invio tutti i byte del campo data del frame.

		USHORT w_ct     = p_frameTx->w_size;
		PUCHAR pby_data = p_frameTx->pby_data;

		for(;w_ct;w_ct--)
		{
		// Leggo dal capo data
			UCHAR by_tmp = *pby_data;

		// Punto al dato sucessivo.
			pby_data++;

		// Scrivo sulla porta.
			if(DeviceIoWrite(by_tmp)!=STATUS_SUCCESS)
			{
#if DBG
				t << "DeviceTxFrame: Error Writing data step " << w_ct << EOL;
#endif
				return	STATUS_UNSUCCESSFUL;
			}

		// Aggiorno il checkSum.
			w_checkSum += by_tmp;
		}

	// 8/9 UCHAR  Invio il checkSum del frame.
		if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
		{
#if DBG
			t << "DeviceTxFrame: Error Writing checksum" << EOL;
#endif
			return	STATUS_UNSUCCESSFUL;
		}

	// Se il checkSum ricevuto e' diverso da quello calcolato
	// setto il codice di ritorno con errore.
		if(w_checkSum != DeviceIoReadW())
		{
#if DBG
			t << "DeviceTxFrame: Error Comparing checksum 2" << EOL;
#endif
			return STATUS_UNSUCCESSFUL;
		}
		else
		{
#if DBG
			t << "DeviceTxFrame: Checksum 2 OK" << EOL;
#endif
			return	STATUS_SUCCESS;
		}
	}

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge/Scrive un frame di IO Diretto da/per la PCFOM usando il protocollo PARBUS
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::DeviceIoDirectFrame(PCCIO_FR_STR*   p_Frame)
{
	NTSTATUS 	status;
	PUCHAR		pby_data; // Puntatore al campo data del frame.
	USHORT		w_memSize; // Dimensione della memoria.
	USHORT		w_checkSum;

	SHORT		s_tmp;
	USHORT		w_tmp;
	UCHAR		by_tmp;
	ULONG		dw_tmp;

	w_checkSum =	p_Frame->HeaderChecksum();

// Prelevo il puntatore al campo data del frame.
	pby_data = p_Frame->pby_data;

// Ricevo dalla scheda Pcc la dimensione della memoria da trasferire.
	w_memSize = p_Frame->w_size;

	switch(p_Frame->by_cmd)
	{
		// Se il comando e' PCCIO_CMD_DIRECT_MEM_RD leggo la memoria RAM

		case PCCIO_CMD_DIRECT_MEM_RD:
		{
		// Per tutto il blocco di memoria da leggere.
			for(;w_memSize > 0;w_memSize--)
			{
			// Ricevo dalla scheda Pcc il byte da leggere.
				if((s_tmp	=	DeviceIoRead())	==	(SHORT)-1)
					return	STATUS_UNSUCCESSFUL;
				
				by_tmp = (UCHAR)(s_tmp & 0xff);

			// Scrivo il byte nel campo data del frame
				*pby_data = by_tmp;
				pby_data++;

			// Sommo al checkSun
				w_checkSum += by_tmp;
			}

		// Ricevo dalla scheda Pcc il checkSum ricevuto.
			if((dw_tmp	=	DeviceIoReadW())	==	(LONG)-1)
				return	STATUS_UNSUCCESSFUL;

			w_tmp = (USHORT)(dw_tmp & 0xffff);

		// Invio alla scheda Pcc il checkSum calcolato del frame.

			if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
				return	STATUS_UNSUCCESSFUL;

		// Controllo se il checkSum ricevuto e' diverso da quello calcolato.
			if(w_checkSum != w_tmp)
				return STATUS_UNSUCCESSFUL;

			break;
		}

		// Se il comando e' PCCIO_CMD_DIRECT_MEM_WR scrivo la memoria RAM
		case	PCCIO_CMD_DIRECT_MEM_WR:
		{
		// Per tutto il blocco di memoria da scrivere.
			for(;w_memSize > 0;w_memSize--)
			{
			// Leggo il byte dal frame
				by_tmp = *pby_data;
				pby_data++;

			// Invio alla scheda Pcc il byte letto dal frame.
				if(DeviceIoWrite(by_tmp)!=STATUS_SUCCESS)
					return	STATUS_UNSUCCESSFUL;

			// Sommo al checkSun
				w_checkSum += by_tmp;
			}

		// Invio alla scheda Pcc il checkSum calcolato del frame.

			if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
				return	STATUS_UNSUCCESSFUL;

		// Controllo se il checkSum ricevuto e' diverso da quello calcolato.
			if(w_checkSum != DeviceIoReadW())
				return STATUS_UNSUCCESSFUL;

			break;
		}
		default:
			return STATUS_UNSUCCESSFUL;
	}

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge un frame di IO Diretto da/per la PCFOM usando il protocollo PARBUS
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

PCCIO_FR_STR*	NTPCFOMDevice::DeviceIoDirectFrameRead(USHORT w_Address, USHORT w_Size)
{
	PCCIO_FR_STR*   p_frame;
	PUCHAR			pby_data; // Puntatore al campo data del frame.
	USHORT			w_memSize; // Dimensione della memoria.
	USHORT			w_checkSum;

	SHORT			s_tmp;
	USHORT			w_tmp;
	UCHAR			by_tmp;

	p_frame		=	new	PCCIO_FR_STR(PCCIO_CMD_DIRECT_MEM_RD, w_Address, w_Size);

	if(p_frame==NULL)
		return	NULL;

	w_checkSum =	p_frame->HeaderChecksum();

// Prelevo il puntatore al campo data del frame.
	pby_data = p_frame->pby_data;

// Ricevo dalla scheda Pcc la dimensione della memoria da trasferire.
	w_memSize = w_Size;

// Per tutto il blocco di memoria da leggere.
	for(;w_memSize > 0;w_memSize--)
	{
	// Ricevo dalla scheda Pcc il byte da leggere.
		if((s_tmp	=	DeviceIoRead())	==	(SHORT)-1)
		{
			delete	p_frame;
			return	NULL;
		}
		
		by_tmp = (UCHAR)(s_tmp & 0xff);

	// Scrivo il byte nel campo data del frame
		*pby_data = by_tmp;
		pby_data++;

	// Sommo al checkSun
		w_checkSum += by_tmp;
	}

// Ricevo dalla scheda Pcc il checkSum ricevuto.
	w_tmp = DeviceIoRead();

// Invio alla scheda Pcc il checkSum calcolato del frame.

	if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
	{
		delete	p_frame;
		return	NULL;
	}

// Controllo se il checkSum ricevuto e' diverso da quello calcolato.
	if(w_checkSum != w_tmp)
	{
		delete	p_frame;
		return NULL;
	}

	return	p_frame;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Scrive un frame di IO Diretto da/per la PCFOM usando il protocollo PARBUS
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::DeviceIoDirectFrameWrite(PCCIO_FR_STR*   p_Frame)
{
	NTSTATUS 	status;
	PUCHAR		pby_data; // Puntatore al campo data del frame.
	USHORT		w_memAdr;  // Indirizzo della memoria.
	USHORT		w_memSize; // Dimensione della memoria.
	USHORT		w_checkSum;

	USHORT		w_tmp;
	UCHAR		by_tmp;

	w_checkSum =	p_Frame->HeaderChecksum();

// Prelevo il puntatore al campo data del frame.
	pby_data = p_Frame->pby_data;

// Ricevo dalla scheda Pcc la dimensione della memoria da trasferire.
	w_memSize = p_Frame->w_size;

	// Se il comando e' PCCIO_CMD_DIRECT_MEM_WR scrivo la memoria RAM

	if(p_Frame->by_cmd	==	PCCIO_CMD_DIRECT_MEM_WR)
	{
	// Per tutto il blocco di memoria da scrivere.
		for(;w_memSize > 0;w_memSize--)
		{
		// Leggo il byte dal frame
			by_tmp = *pby_data;
			pby_data++;

		// Invio alla scheda Pcc il byte letto dal frame.
			if(DeviceIoWrite(by_tmp)!=STATUS_SUCCESS)
				return	STATUS_UNSUCCESSFUL;

		// Sommo al checkSun
			w_checkSum += by_tmp;
		}

	// Invio alla scheda Pcc il checkSum calcolato del frame.

		if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
			return	STATUS_UNSUCCESSFUL;

	// Controllo se il checkSum ricevuto e' diverso da quello calcolato.
		if(w_checkSum != DeviceIoReadW())
			return STATUS_UNSUCCESSFUL;

		return STATUS_SUCCESS;
	}

	return STATUS_UNSUCCESSFUL;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Riceve un frame dalla PCFOM usando il protocollo PARBUS
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


PCCIO_FR_STR*	NTPCFOMDevice::DeviceRxFrame()
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameRx;
	PUCHAR			pby_data; // Puntatore al campo data del frame.
	USHORT			w_memAdr;  // Indirizzo della memoria.
	USHORT			w_memSize; // Dimensione della memoria.

	SHORT			s_tmp;
	USHORT			w_tmp;
	UCHAR			by_tmp;
	UCHAR			by_cmd;
	USHORT			w_cmdArg;
	USHORT			w_dataSize;
	USHORT			w_checkSum;

	//	Byte 0 ricevo il carattere di sincronismo inizio comunicazione
	//	se e' diverso da 0x55 ritorno errore

	if(DeviceIoRead()	!=	0x55)
		return NULL;

	//	Byte 1 ricevo il comando

	by_cmd	=	(UCHAR)(DeviceIoRead() & 0xff);

	//	Byte 2 e 3 ricevo l'argomento del comando

	w_cmdArg	=	(USHORT)(DeviceIoReadW() & 0xffff);

	//	Byte 4 e 5 ricevo la dimensione del frame

	w_dataSize	=	(USHORT)(DeviceIoReadW() & 0xffff);

	//	Byte 6 e 7 ricevo il checksum dell'header frame
	//	In pratica il checksum del frame senza campo data
	//	Viene usato per verificare il comando prima che venga eseguito

	w_checkSum	=	(USHORT)(DeviceIoReadW() & 0xffff);


	//	In funzione del tipo di comando creo il frame da ricevere
	//	Se il comando e' <= di PCCIO_CMD_DIRECT_END

	if(by_cmd >	PCCIO_CMD_DIRECT_END)
/*
	{
		p_frameRx	=	new	PCCIO_FR_STR(by_cmd, w_cmdArg, 0);

		p_frameRx->w_size	=	w_dataSize;
	}
	else
*/
		p_frameRx	=	new	PCCIO_FR_STR(by_cmd, w_cmdArg, w_dataSize);

	if(p_frameRx == NULL)
	{
		DeviceIoWrite((UCHAR)0x00);

		//	Lunghezza superiore alla massima consentita
#if DBG
		t << "p_frame == NULL" << EOL;
#endif
		return NULL;
	}

// Calcolo il checkSum dell'header.
	w_tmp =	p_frameRx->HeaderChecksum();

// Invio il checkSum dell'header calcolato.
	if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
		return NULL;

// Se il checkSum dell'header ricevuto e' diverso da quello calcolato
	if(w_checkSum != w_tmp)
	{
	// Distruggo il frame ricevuto.
		delete p_frameRx;

	// Esco con errore.
		return NULL;
	}

// Se il comando e' <= di PCCIO_CMD_DIRECT_END
	if(by_cmd <= PCCIO_CMD_DIRECT_END)
	{
	// Chiamo la macro specializzata per ricevere i frame direttamente.
		switch(by_cmd)
		{
			case	PCCIO_CMD_DIRECT_MEM_RD:
				return DeviceIoDirectFrameRead(w_cmdArg, w_dataSize);
			default:
				break;
		}

		return NULL;
	}
	else if(w_dataSize)
	{

	// Prelevo il puntatore al campo data del frame.
		pby_data = p_frameRx->pby_data;

	// Per tutto il blocco di memoria da leggere.
		for(;w_dataSize > 0;w_dataSize--)
		{
		// Ricevo dalla scheda Pcc il byte da leggere.
			if((s_tmp	=	DeviceIoRead())	==	(SHORT)-1)
			{
				delete	p_frameRx;
				return	NULL;
			}

			by_tmp = (UCHAR)(s_tmp & 0xff);

		// Scrivo il byte nel campo data del frame
			*pby_data = by_tmp;
			pby_data++;

		// Sommo al checkSun
			w_checkSum += by_tmp;
		}

	// Ricevo dalla scheda Pcc il checkSum ricevuto.
		w_tmp = (USHORT)(DeviceIoReadW() & 0xffff);

	// Invio alla scheda Pcc il checkSum calcolato del frame.
		if(DeviceIoWrite(w_checkSum)!=STATUS_SUCCESS)
			return	NULL;

	// Controllo se il checkSum ricevuto e' diverso da quello calcolato.
		if(w_checkSum != w_tmp)
		{
			delete	p_frameRx;
			return 	NULL;
		}

		return	p_frameRx;
	}

	return	NULL;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Funzione chiamata per leggere la configurazione della scheda PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::GetConfig()
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_TRACEW_CFG,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti

				// Stato abilitazione trace.
					by_channelEnable[0]	= (p_frameRx->w_arg&0x0001) ? TRUE : FALSE;
					by_channelEnable[1]	= (p_frameRx->w_arg&0x0002) ? TRUE : FALSE;
					by_channelEnable[2]	= (p_frameRx->w_arg&0x0004) ? TRUE : FALSE;
					by_channelEnable[3]	= (p_frameRx->w_arg&0x0008) ? TRUE : FALSE;

				// Indirizzi trace.
					w_address[0] 	= SWAPWORD(pw_tmp[0]);	//	H8_TO_INTEL_W(pw_tmp[0]);
					w_address[1] 	= SWAPWORD(pw_tmp[1]);	//	H8_TO_INTEL_W(pw_tmp[1]);
					w_address[2] 	= SWAPWORD(pw_tmp[2]);	//	H8_TO_INTEL_W(pw_tmp[2]);
					w_address[3]	= SWAPWORD(pw_tmp[3]);	//	H8_TO_INTEL_W(pw_tmp[3]);

				// Dimensione trace.
					w_size[0]	= SWAPWORD(pw_tmp[4]);	//	H8_TO_INTEL_W(pw_tmp[4]);
					w_size[1]	= SWAPWORD(pw_tmp[5]);	//	H8_TO_INTEL_W(pw_tmp[5]);
					w_size[2]	= SWAPWORD(pw_tmp[6]);	//	H8_TO_INTEL_W(pw_tmp[6]);
					w_size[3]	= SWAPWORD(pw_tmp[7]);	//	H8_TO_INTEL_W(pw_tmp[7]);

				// Numero elementi per calcolo della media.
					w_averageSize[0]	= SWAPWORD(pw_tmp[8]);	//	H8_TO_INTEL_W(pw_tmp[8]);
					w_averageSize[1]	= SWAPWORD(pw_tmp[9]);	//	H8_TO_INTEL_W(pw_tmp[9]);
					w_averageSize[2]	= SWAPWORD(pw_tmp[10]);	//	H8_TO_INTEL_W(pw_tmp[10]);
					w_averageSize[3]	= SWAPWORD(pw_tmp[11]);	//	H8_TO_INTEL_W(pw_tmp[11]);

					delete	p_frameRx;
				}
			}
		}
	}

	delete	p_frameTx;

	return status;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede un singolo valore alla PCFOM con il frame PCCFOM_CMD_GET_SINGLE_VALUE
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::GetSingleMediaSum(USHORT	w_Index, PULONG pdw_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;

#if DBG
	t << "Entering GetSingleMediaSum" << EOL;
#endif
// Creo un frame per richiedere la configurazione dei trace.

	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_MEDIA_SUM,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

#if DBG
	t << "DeviceTxFrame" << EOL;
#endif

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

#if DBG
		t << "DecodeRx" << EOL;
#endif
		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();
#if DBG
			t << "IsFrameAvailable" << EOL;
#endif

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();
#if DBG
				t << "DeviceRxFrame" << EOL;
#endif
				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti
					*pdw_Value	=	MAKELONG(SWAPWORD(pw_tmp[0]), SWAPWORD(pw_tmp[1])) ;
	

					// Distruggo il frame ricevuto.

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSingleValue	with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede un singolo valore alla PCFOM con il frame PCCFOM_CMD_GET_SINGLE_VALUE
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::GetSingleValue(USHORT	w_Index, USHORT* pw_Value, USHORT* pw_Average)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;

#if DBG
	t << "Entering GetSingleValue " << EOL;
#endif
// Creo un frame per richiedere la configurazione dei trace.

	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_VALUE,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

#if DBG
	t << "DeviceTxFrame" << EOL;
#endif

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

#if DBG
		t << "DecodeRx" << EOL;
#endif
		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();
#if DBG
			t << "IsFrameAvailable" << EOL;
#endif

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();
#if DBG
				t << "DeviceRxFrame" << EOL;
#endif
				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti
					*pw_Value	=	SWAPWORD(pw_tmp[0]);
					
					*pw_Average	=	SWAPWORD(pw_tmp[1]);

					// Distruggo il frame ricevuto.

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSingleValue	with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_TRACEW_VALUE
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::GetTraceWValue() // Obsolete
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;

#if DBG
	t << "Entering GetMultipleValue " << EOL;
#endif
// Creo un frame per richiedere la configurazione dei trace.

	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_TRACEW_VALUE,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

#if DBG
	t << "DeviceTxFrame" << EOL;
#endif

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

#if DBG
		t << "DecodeRx" << EOL;
#endif
		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();
#if DBG
			t << "IsFrameAvailable" << EOL;
#endif

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();
#if DBG
				t << "DeviceRxFrame" << EOL;
#endif
				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti
					w_value[0] = SWAPWORD(pw_tmp[0]);
					w_value[1] = SWAPWORD(pw_tmp[1]);
					w_value[2] = SWAPWORD(pw_tmp[2]);
					w_value[3] = SWAPWORD(pw_tmp[3]);
					
#if DBG
					t << "w_value[0]" << w_value[0] << EOL;
					t << "w_value[1]" << w_value[1] << EOL;
					t << "w_value[2]" << w_value[2] << EOL;
					t << "w_value[3]" << w_value[3] << EOL;
#endif
					w_average[0]	= SWAPWORD(pw_tmp[4]);
					w_average[1]	= SWAPWORD(pw_tmp[5]);
					w_average[2]	= SWAPWORD(pw_tmp[6]);
					w_average[3]	= SWAPWORD(pw_tmp[7]);

#if DBG
					t << "w_average[0]" << w_average[0]	<< EOL;
					t << "w_average[1]" << w_average[1]	<< EOL;
					t << "w_average[2]" << w_average[2]	<< EOL;
					t << "w_average[3]" << w_average[3]	<< EOL;
#endif

					// Distruggo il frame ricevuto.

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetMultipleValue with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_TRACEW_VALUE
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::GetMultipleValues(PUSHORT pw_Values, PUSHORT pw_Avgerages)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;

#if DBG
	t << "Entering GetMultipleValue " << EOL;
#endif
// Creo un frame per richiedere la configurazione dei trace.

	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_MULTIPLE_VALUE,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

#if DBG
	t << "DeviceTxFrame" << EOL;
#endif

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

#if DBG
		t << "DecodeRx" << EOL;
#endif
		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();
#if DBG
			t << "IsFrameAvailable" << EOL;
#endif

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();
#if DBG
				t << "DeviceRxFrame" << EOL;
#endif
				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti
					pw_Values[0]	= SWAPWORD(pw_tmp[0]);
					pw_Values[1]	= SWAPWORD(pw_tmp[1]);
					pw_Values[2]	= SWAPWORD(pw_tmp[2]);
					pw_Values[3]	= SWAPWORD(pw_tmp[3]);
					pw_Values[4]	= SWAPWORD(pw_tmp[4]);
					pw_Values[5]	= SWAPWORD(pw_tmp[5]);
					pw_Values[6]	= SWAPWORD(pw_tmp[6]);
					pw_Values[7]	= SWAPWORD(pw_tmp[7]);
					

					pw_Avgerages[0]	= SWAPWORD(pw_tmp[8]);
					pw_Avgerages[1]	= SWAPWORD(pw_tmp[9]);
					pw_Avgerages[2]	= SWAPWORD(pw_tmp[10]);
					pw_Avgerages[3]	= SWAPWORD(pw_tmp[11]);
					pw_Avgerages[4]	= SWAPWORD(pw_tmp[12]);
					pw_Avgerages[5]	= SWAPWORD(pw_tmp[13]);
					pw_Avgerages[6]	= SWAPWORD(pw_tmp[14]);
					pw_Avgerages[7]	= SWAPWORD(pw_tmp[15]);
					
					// Distruggo il frame ricevuto.

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetMultipleValue with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Sincronizza il timer di acquisizione del campioni FOM all'interno della PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::ResetTracesTimer()
{
	PC_PCCFOM_TRACE_TIMER_RESET;

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetSampleXSUM(PUSHORT pw_sampleXSumCt, PUINT128 puint128_XSumCH0_01, PUINT128 puint128_XSumCH1_01)	//	Obsolete
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;
	UINT128			ui128_tmp;

#if DBG
	t << "Entering GetSampleXSum " << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SAMPLE_XSUM,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				//	Leggo i valori diretti e medie

					GetTraceWValue();

				// Swap e salvo i dati ricevuti

					w_sampleXSumCt = SWAPWORD(pw_tmp[0]);

					if(w_value[0]>0)
					{
						UINT128Zero(&ui128_tmp);

						ui128_tmp.data[0] = SWAPWORD(pw_tmp[3]);
						ui128_tmp.data[1] = SWAPWORD(pw_tmp[2]);
						ui128_tmp.data[2] = SWAPWORD(pw_tmp[1]);

						UINT128SumUINT128(&ui128_sampleXSumCH0_01,&ui128_tmp);
						*puint128_XSumCH0_01	=	ui128_sampleXSumCH0_01;

/*
						if(w_sampleXSumCt);
							f_sampleXMedia01 = UINT128ToDouble(&ui128_tmp) / (double)w_sampleXSumCt;
						else
							f_sampleXMedia01 = 0;
*/
					}
/*
					else
						f_sampleXMedia01 = 0;
*/
					if(w_value[2] > 0)
					{
						UINT128Zero(&ui128_tmp);

						ui128_tmp.data[0] = SWAPWORD(pw_tmp[6]);
						ui128_tmp.data[1] = SWAPWORD(pw_tmp[5]);
						ui128_tmp.data[2] = SWAPWORD(pw_tmp[4]);

						UINT128SumUINT128(&ui128_sampleXSumCH1_01,&ui128_tmp);
						*puint128_XSumCH1_01	=	ui128_sampleXSumCH0_01;
/*
						if(w_sampleXSumCt);
							f_sampleXMedia23 = UINT128ToDouble(&ui128_tmp) / (double)w_sampleXSumCt;
						else
							f_sampleXMedia23 = 0;
*/
					}
/*
					else
						f_sampleXMedia23 = 0;
*/
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSampleXSum with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Imposta il valore dei flag di inversione dei singoli canali PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::SetTraceInvert()
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	USHORT          w_tmp;

// Formo l'argomento del frame da mandare alla scheda PccFom.
	w_tmp = 0;

	if(by_invertEnable[0])
		w_tmp	|=	0x0001;

	if(by_invertEnable[1])
		w_tmp	|=	0x0002;

	if(by_invertEnable[2])
		w_tmp	|=	0x0004;

	if(by_invertEnable[3])
		w_tmp	|=	0x0008;

// Creo un frame per impostare l'abilitazione
// all'inversione del valore campionato.

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_INVERT_CH_SET,w_tmp,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

	status	=	DeviceTxFrame(p_frameTx);

	delete	p_frameTx;

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge  i valore dei flag di inversione dei singoli canali PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetTraceInvert()
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_INVERT_CH_TST,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.

					by_invertEnable[0] = (p_frameRx->w_arg&0x0001) ? TRUE : FALSE;
					by_invertEnable[1] = (p_frameRx->w_arg&0x0002) ? TRUE : FALSE;
					by_invertEnable[2] = (p_frameRx->w_arg&0x0004) ? TRUE : FALSE;
					by_invertEnable[3] = (p_frameRx->w_arg&0x0008) ? TRUE : FALSE;

					delete	p_frameRx;
				}
			}
		}
	}

	delete	p_frameTx;

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge il valore dei flag di abilitazione del trace dei singoli canali PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetTraceEnable()
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_TRACEW_EN_TST,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.

					by_channelEnable[0] = (p_frameRx->w_arg&0x0001) ? TRUE : FALSE;
					by_channelEnable[1] = (p_frameRx->w_arg&0x0002) ? TRUE : FALSE;
					by_channelEnable[2] = (p_frameRx->w_arg&0x0004) ? TRUE : FALSE;
					by_channelEnable[3] = (p_frameRx->w_arg&0x0008) ? TRUE : FALSE;

					delete	p_frameRx;
				}
			}
		}
	}

	delete	p_frameTx;

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Imposta il valore dei flag di abilitazione del trace dei singoli canali PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::SetTraceEnable()
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

	for(int i=0;i<PCFOM_MAX_CHANNELS;i++)
	{
		if(by_channelEnable[i])
			w_tmp	|=	(1	<<	i);
	}

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_TRACEW_EN_SET,w_tmp,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	delete	p_frameTx;

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Dato un TraceID lo validifica secondo la disponibilita' (predefinta) del TRACE
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	 NTPCFOMDevice::ValidateTraceID(UCHAR by_TraceCh)
{
	switch(by_TraceCh)
	{
		case	PCCFOM_CH0_0:
			return STATUS_SUCCESS;

		case	PCCFOM_CH0_1:
			return STATUS_SUCCESS;
		
		case	PCCFOM_CH0_2:
			return STATUS_UNSUCCESSFUL;

		case	PCCFOM_CH0_3:	
			return STATUS_UNSUCCESSFUL;

		case	PCCFOM_CH1_0:
			return STATUS_SUCCESS;

		case	PCCFOM_CH1_1:
			return STATUS_SUCCESS;

		case	PCCFOM_CH1_2:
			return STATUS_UNSUCCESSFUL;

		case	PCCFOM_CH1_3:
			return STATUS_UNSUCCESSFUL;

		default:
			break;
	}
	
	return STATUS_UNSUCCESSFUL;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  imposta il valore dei flag di abilitazione del trace di un singolo canale PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::SetTraceEnable(UCHAR by_TraceCh)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

	if(!NT_SUCCESS(ValidateTraceID(by_TraceCh)))
		return STATUS_UNSUCCESSFUL;
	
	for(int i=0;i<PCFOM_MAX_CHANNELS;i++)
	{
		if(by_channelEnable[i])
			w_tmp	|=	(1	<<	i);
	}

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_TRACEW_EN_SET,w_tmp,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	delete	p_frameTx;

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Legge il valore di un Trace dalla PCFOM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::GetTrace(UCHAR by_TraceCh, PNTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER	pValue)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;
	UCHAR			by_traceEn;
	UCHAR*			pby_h8MemoryBuffer;
	UCHAR*			pby_localMemoryBuffer;

	status		=	STATUS_SUCCESS;

	if(!NT_SUCCESS(ValidateTraceID(by_TraceCh)))
		return STATUS_UNSUCCESSFUL;

	GetTraceEnable();

	by_traceEn	=	by_channelEnable[by_TraceCh];

	by_channelEnable[by_TraceCh]	=	FALSE;

	status	=	SetTraceEnable();

	USHORT	w_len	=	sizeof(PC_TRACE_H8300_STR) + w_size[by_TraceCh] * sizeof(USHORT);

	pby_h8MemoryBuffer	=	new	UCHAR[w_len];

	if(pby_h8MemoryBuffer==NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Error allocating memory H8 TRACE memory buffer!" << EOL;
#endif
		return STATUS_UNSUCCESSFUL;			
	}

	pby_localMemoryBuffer	=	new	UCHAR[w_len];

	if(pby_localMemoryBuffer==NULL)
	{
		//	Lunghezza superiore alla massima consentita 
#if DBG
		t << "Error allocating memory H8 TRACE memory buffer!" << EOL;
#endif
		delete[]	pby_h8MemoryBuffer;

		return	STATUS_UNSUCCESSFUL;			
	}

	RtlZeroMemory(pby_h8MemoryBuffer, w_len);
	RtlZeroMemory(pby_localMemoryBuffer, w_len);

	status	=	DeviceIoMemRead(pby_h8MemoryBuffer, w_address[by_TraceCh], w_len);

	if(!NT_SUCCESS(status))
	{
#if DBG
		t << "Error reading trace from PCFOM memory!" << EOL;
#endif
		delete[]	pby_h8MemoryBuffer;
		delete[]	pby_localMemoryBuffer;
		
		return	status;
	}

	PC_TRACE_H8300_STR*	p_h8300Trace	=	reinterpret_cast<PC_TRACE_H8300_STR*>(pby_h8MemoryBuffer);
	PC_TRACE_PC_STR*	p_localTrace	=	reinterpret_cast<PC_TRACE_PC_STR*>(pby_localMemoryBuffer);

	ConvertH8300ToPCTrace(p_h8300Trace, p_localTrace);

	ConvertPCTraceToIoctlBuffer(p_localTrace, pValue);

	delete[]	pby_h8MemoryBuffer;
	delete[]	pby_localMemoryBuffer;
		
	by_channelEnable[by_TraceCh]	=	by_traceEn;

	status	=	SetTraceEnable();

	return	status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Converte il valore di una trace dal formato H8300 al formato PC
 *		  In pratica converte i valori del buffer da BIG_ENDIAN a LITTLE_ENDIAM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::ConvertH8300ToPCTrace(PC_TRACE_H8300_STR* p_TraceH8300,// Punt.trace H8300.
													PC_TRACE_PC_STR* p_Trace)     // Punt.trace PC.
{
	USHORT 		w_H8300FirstItemAdr;
	USHORT 		w_tmp;

	USHORT* 	pw_H8300TraceBuffer=NULL;
	USHORT* 	pw_TraceBuffer=NULL;

	USHORT* 	pw_averageOldItem=NULL;// Puntatore elemento piu' vecchio della sommatoria.
	USHORT* 	pw_itemFirst=NULL;   // Puntatore al primo elemento del trace.
	USHORT* 	pw_itemLast=NULL;    // Puntatore all'ultimo elemento del trace.
	USHORT* 	pw_itemIns=NULL;     // Puntatore all'ultimo elemento inserito nel trace.

// Esco se i puntatori non esistono.
	if(p_TraceH8300	==	NULL || p_Trace	==	NULL)
		return	STATUS_UNSUCCESSFUL;

// Flag di abilitazione.
	p_Trace->by_en = p_TraceH8300->by_en;

// Flag ultimo stato abilitazione,(ad uso interno).
	p_Trace->by_lastEn = p_TraceH8300->by_lastEn;

// Contiene l'ora di abilitazione del trace.
// Il tempo viene espresso in  in sample count.
	p_Trace->dw_enTime = H8_TO_INTEL_DW(p_TraceH8300->dw_enTime);

// Contiene l'ora di disabilitazione del trace.
// Il tempo viene espresso in  in sample count.
	p_Trace->dw_disTime = H8_TO_INTEL_DW(p_TraceH8300->dw_disTime);

// Dimensione del trace.
	p_Trace->w_size = H8_TO_INTEL_W(p_TraceH8300->w_size);

// Contatore elementi inseriti nel trace.
	p_Trace->w_itemCt = H8_TO_INTEL_W(p_TraceH8300->w_itemCt);

// Valore dell'ultimo elementi inserito nel trace.
	p_Trace->w_value = H8_TO_INTEL_W(p_TraceH8300->w_value);

// Numero elementi per il calcolo della media.
	p_Trace->w_averageSize = H8_TO_INTEL_W(p_TraceH8300->w_averageSize);

// Sommatoria degli elementi per il calcolo della media.
	p_Trace->dw_averageSum = H8_TO_INTEL_DW(p_TraceH8300->dw_averageSum);

// Contatore elementi inseriti nella sommatoria.
	p_Trace->w_averageItemCt = H8_TO_INTEL_W(p_TraceH8300->w_averageItemCt);

// Puntatore al primo elemento del trace.
	pw_itemFirst = (USHORT*)(&((UCHAR*)p_Trace)[sizeof(PC_TRACE_PC_STR)]);

// Puntatore elemento piu' vecchio della sommatoria.
	w_H8300FirstItemAdr = H8_TO_INTEL_W(p_TraceH8300->pw_itemFirst);
	w_tmp               = H8_TO_INTEL_W(p_TraceH8300->pw_averageOldItem);
	w_tmp              -= w_H8300FirstItemAdr;
	w_tmp              /= sizeof(USHORT);

	pw_averageOldItem = &pw_itemFirst[w_tmp];

// Puntatore all'ultimo elemento del trace.
	pw_itemLast     	= &pw_itemFirst[p_Trace->w_size-1];

// Puntatore all'ultimo elemento inserito nel trace.
	w_tmp      = H8_TO_INTEL_W(p_TraceH8300->pw_itemIns);
	w_tmp     -= w_H8300FirstItemAdr;
	w_tmp     /= sizeof(USHORT);

	pw_itemIns = &pw_itemFirst[w_tmp];

// Copio tutto il trace buffer.
	pw_TraceBuffer      = pw_itemFirst;
	pw_H8300TraceBuffer = (USHORT*)(&((UCHAR*)p_TraceH8300)[sizeof(PC_TRACE_H8300_STR)]);

	for(w_tmp = p_Trace->w_size;w_tmp;w_tmp--)
	{
		*pw_TraceBuffer = H8_TO_INTEL_W(*pw_H8300TraceBuffer);
		 pw_TraceBuffer++;
		 pw_H8300TraceBuffer++;
	}

// Salvo i puntatori
	p_Trace->pw_averageOldItem = pw_averageOldItem;
	p_Trace->pw_itemFirst    = pw_itemFirst;
	p_Trace->pw_itemLast     = pw_itemLast;
	p_Trace->pw_itemIns      = pw_itemIns;

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Converte il valore di una trace dal formato PC a IOCTL
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::ConvertPCTraceToIoctlBuffer(PC_TRACE_PC_STR*	p_Trace, NTPCFOM_IOCTL_CHANNEL_TRACE_BUFFER* p_TraceBuffer)
{
	p_TraceBuffer->by_En			=	p_Trace->by_en;
	p_TraceBuffer->dw_DisTime		=	p_Trace->dw_disTime;
	p_TraceBuffer->dw_EnTime		=	p_Trace->dw_enTime;
	p_TraceBuffer->dw_AverageSum	=	p_Trace->dw_averageSum;
	p_TraceBuffer->w_ItemCt			=	p_Trace->w_itemCt;
	p_TraceBuffer->w_AverageItemCt	=	p_Trace->w_averageItemCt;
	p_TraceBuffer->w_AverageSize	=	p_Trace->w_averageSize;
	p_TraceBuffer->w_Size			=	p_Trace->w_size;
	p_TraceBuffer->w_Value			=	p_Trace->w_value;

	if(p_Trace->pw_itemIns==p_Trace->pw_itemLast)
	{
		RtlCopyMemory(p_TraceBuffer->pw_Items, p_Trace->pw_itemFirst, p_Trace->w_size * sizeof(USHORT));
	}
	else
	{
		LONG	l_tmp;

		l_tmp	=	p_Trace->pw_itemLast	-	p_Trace->pw_itemIns;
		
		RtlCopyMemory(p_TraceBuffer->pw_Items, p_Trace->pw_itemIns+1, l_tmp * sizeof(USHORT));
		
		USHORT*	pw_lastItem	=	&p_TraceBuffer->pw_Items[l_tmp];

		l_tmp	=	p_Trace->pw_itemIns	-	p_Trace->pw_itemFirst;
		l_tmp++;

		RtlCopyMemory(pw_lastItem+1, p_Trace->pw_itemFirst, l_tmp * sizeof(USHORT));
	}

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Ritorna il Firmware ID della scheda
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	NTPCFOMDevice::GetFirmwareID(PUCHAR	pc_Buffer)
{
	NTSTATUS 		status;

	status		=	DeviceIoMemRead(pc_Buffer, PCFOM_REVISION_ID_ADDRESS, NTPCFOM_FIRMWARE_ID_MAX_LEN);

	return	status;
}


NTSTATUS	NTPCFOMDevice::GetFirmwareVersion(PUSHORT	pw_MajorVersion, PUSHORT	pw_MinorVersion)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;
	UINT128			ui128_tmp;

#if DBG
	t << "Entering GetFirmwareVersion" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_FIRMWARE_VERSION,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

					*pw_MajorVersion	=	SWAPWORD(pw_tmp[0]);
					
					*pw_MinorVersion	=	SWAPWORD(pw_tmp[1]);

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetFirmwareVersion with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}
	

NTSTATUS	NTPCFOMDevice::SetSamplingDivisorValue(USHORT	w_Index, USHORT	w_SamplingDivisorValue)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_SAMPLING_DIVISOR_VALUE,w_Index,2);

	*(USHORT*)p_frameTx->pby_data	=	SWAPWORD(w_SamplingDivisorValue);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp==-1)
					{

						delete	p_frameRx;
						delete	p_frameTx;

						return STATUS_UNSUCCESSFUL;
					}
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting SetSingleTraceInvert with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


NTSTATUS	NTPCFOMDevice::GetSamplingDivisorValue(USHORT	w_Index, PUSHORT	pw_SamplingDivisorValue)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SAMPLING_DIVISOR_VALUE,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.

					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp!=-1)
						*pw_SamplingDivisorValue	=	w_tmp;
			

					delete	p_frameRx;
				}
			}
		}
	}

	delete	p_frameTx;

	return STATUS_SUCCESS;

}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede una singol misura XSUM PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetSingleXSUMValue(USHORT w_Index, PUSHORT pw_sampleXSumCt, PUINT128 puint128_XSum)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;
	UINT128			ui128_tmp;
	USHORT			w_values[2];

#if DBG
	t << "GetSingleXSUMValue" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_XSUM_VALUE,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti

					w_sampleXSumCt = SWAPWORD(pw_tmp[0]);

					*pw_sampleXSumCt	=	w_sampleXSumCt;

					w_values[0]	= SWAPWORD(pw_tmp[1]);
					w_values[1]	= SWAPWORD(pw_tmp[2]);

					if(w_value[0]>0 && w_values[1]>0)
					{
						UINT128Zero(&ui128_tmp);

						switch(w_Index)
						{
							case	PCCFOM_XSUM_CH0_01:
						
								ui128_tmp.data[0] = SWAPWORD(pw_tmp[5]);
								ui128_tmp.data[1] = SWAPWORD(pw_tmp[4]);
								ui128_tmp.data[2] = SWAPWORD(pw_tmp[3]);

								UINT128SumUINT128(&ui128_sampleXSumCH0_01,&ui128_tmp);
								break;
							case	PCCFOM_XSUM_CH1_01:
								ui128_tmp.data[0] = SWAPWORD(pw_tmp[6]);
								ui128_tmp.data[1] = SWAPWORD(pw_tmp[5]);
								ui128_tmp.data[2] = SWAPWORD(pw_tmp[4]);

								UINT128SumUINT128(&ui128_sampleXSumCH1_01,&ui128_tmp);
								break;
							default:
								break;
						}

					}

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSampleXSum with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede una singola configurazione dato un canale
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetSingleTraceConfig(USHORT w_Index, PNTPCFOM_IOCTL_GET_SINGLE_TRACE_CONFIG_BUFFER	pBuffer)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;
	UINT128			ui128_tmp;
	USHORT			w_values[2];

#if DBG
	t << "GetSingleTraceConfig" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_TRACE_CONFIG,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

				// Swap e salvo i dati ricevuti

					pBuffer->w_Address			=	SWAPWORD(pw_tmp[0]);        // Indirizzo trace nella memoria della scheda PccFom.
					pBuffer->w_Size				=	SWAPWORD(pw_tmp[1]);       	// Dimensione del trace.
					pBuffer->w_AverageSize		=	SWAPWORD(pw_tmp[2]);  		// Numero elementi per il calcolo della media.
					pBuffer->w_SamplingDivisor	=	SWAPWORD(pw_tmp[3]);	// Valore del divisore rispetto alla frequenza di campionamento
					pw_tmp[4]	=	SWAPWORD(pw_tmp[4]);

					pBuffer->by_InvertEnable		=	LOBYTE(pw_tmp[4]);  	// Abilitazione alla inversione del valore campionato.
					pBuffer->by_ChannelEnable	=	HIBYTE(pw_tmp[4]);	// Flag di abilitazione.

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSampleXSum with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}
	


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetMultipleXSUMValues(PUSHORT pw_sampleXSumCt, PUINT128 puint128_XSumCH0_01, PUINT128 puint128_XSumCH1_01)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT*         pw_tmp;
	NTSTATUS 		status;
	UINT128			ui128_tmp[2];
	USHORT			w_values[4];
	
#if DBG
	t << "Entering GetSampleXSum " << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_MULTIPLE_XSUM_VALUE,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.
					pw_tmp = (USHORT*)p_frameRx->pby_data;

					w_sampleXSumCt = SWAPWORD(pw_tmp[0]);

					*pw_sampleXSumCt	=	w_sampleXSumCt;

					w_values[0]	= SWAPWORD(pw_tmp[1]);
					w_values[1]	= SWAPWORD(pw_tmp[2]);

					if(w_value[0]>0 && w_values[1]>0)
					{
						UINT128Zero(&ui128_tmp[0]);

						ui128_tmp[0].data[0] = SWAPWORD(pw_tmp[5]);
						ui128_tmp[0].data[1] = SWAPWORD(pw_tmp[4]);
						ui128_tmp[0].data[2] = SWAPWORD(pw_tmp[3]);

						UINT128SumUINT128(&ui128_sampleXSumCH0_01,&ui128_tmp[0]);
					}		
						
					w_values[2]	= SWAPWORD(pw_tmp[6]);
					w_values[3]	= SWAPWORD(pw_tmp[7]);

					if(w_value[2]>0 && w_values[3]>0)
					{
						UINT128Zero(&ui128_tmp[1]);

						ui128_tmp[1].data[0] = SWAPWORD(pw_tmp[10]);
						ui128_tmp[1].data[1] = SWAPWORD(pw_tmp[9]);
						ui128_tmp[1].data[2] = SWAPWORD(pw_tmp[8]);

						UINT128SumUINT128(&ui128_sampleXSumCH1_01,&ui128_tmp[1]);
					}

					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSampleXSum with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetSingleTraceEnable(USHORT	w_Index, PUSHORT	pw_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetSingleTraceEnable" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_TRACE_ENABLE,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp==-1)
					{

						delete	p_frameRx;
						delete	p_frameTx;

						return STATUS_UNSUCCESSFUL;
					}

					*pw_Value	=	w_tmp ?	TRUE	:	FALSE;
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSingleTraceEnable with Success" << EOL;
#endif

	return STATUS_SUCCESS;

}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::SetSingleTraceEnable(USHORT	w_Index, USHORT	w_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetSingleTraceEnable" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_SINGLE_TRACE_ENABLE,w_Index,2);

	*(USHORT*)p_frameTx->pby_data	=	SWAPWORD(w_Value);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp==FALSE)
					{

						delete	p_frameRx;
						delete	p_frameTx;

						return STATUS_UNSUCCESSFUL;
					}
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting SetSingleTraceEnable with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::GetSingleTraceInvert(USHORT	w_Index, PUSHORT	pw_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetSingleTraceInvert" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_SINGLE_TRACE_INVERT,w_Index,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp==-1)
					{

						delete	p_frameRx;
						delete	p_frameTx;

						return STATUS_UNSUCCESSFUL;
					}

					*pw_Value	=	w_tmp ?	TRUE	:	FALSE;
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetSingleTraceInvert with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}


/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_SAMPLE_XSUM
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS	 NTPCFOMDevice::SetSingleTraceInvert(USHORT	w_Index, USHORT	w_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetSingleTraceInvert" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_SINGLE_TRACE_INVERT,w_Index,2);

	*(USHORT*)p_frameTx->pby_data	=	SWAPWORD(w_Value);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					if(w_tmp==FALSE)
					{

						delete	p_frameRx;
						delete	p_frameTx;

						return STATUS_UNSUCCESSFUL;
					}
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting SetSingleTraceInvert with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS		NTPCFOMDevice::GetMultipleTraceEnable(PUSHORT	pw_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetMultipleTraceEnable" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_MULTIPLE_TRACE_ENABLE,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					*pw_Value	=	w_tmp;
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetMultipleTraceEnable with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS		NTPCFOMDevice::SetMultipleTraceEnable(USHORT	w_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering SetMultipleTraceEnable" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_MULTIPLE_TRACE_ENABLE,SWAPWORD(w_Value),0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting SetMultipleTraceEnable with Success" << EOL;
#endif

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS		NTPCFOMDevice::GetMultipleTraceInvert(PUSHORT	pw_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering GetMultipleTraceInvert" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_MULTIPLE_TRACE_INVERT,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					*pw_Value	=	w_tmp;
	
					delete	p_frameRx;
				}
			}
		}
	}

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting GetMultipleTraceEnable with Success" << EOL;
#endif

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

NTSTATUS		NTPCFOMDevice::SetMultipleTraceInvert(USHORT	w_Value)
{
	PCCIO_FR_STR*   p_frameTx = NULL;
	USHORT			w_tmp;
	NTSTATUS 		status;
	
#if DBG
	t << "Entering SetMultipleTraceInvert" << EOL;
#endif

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_MULTIPLE_TRACE_INVERT,SWAPWORD(w_Value),0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

// Distruggo il frame trasmesso.

	delete	p_frameTx;

#if DBG
	t << "Exiting SetMultipleTraceInvert with Success" << EOL;
#endif

	return status;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::GetFOMMode(PUSHORT	pw_FOMMode)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_GET_FOM_MODE,0,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	if(NT_SUCCESS(status))
	{
		status	=	DecodeRx(p_frameTx);

		if(NT_SUCCESS(status))
		{
			status	=	IsFrameAvailable();

			if(NT_SUCCESS(status))
			{
				p_frameRx	=	DeviceRxFrame();

				if(p_frameRx!=NULL)
				{
				// Punto il campo data del frame di risposta.

					w_tmp	=	SWAPWORD(p_frameRx->w_arg);

					*pw_FOMMode	=	w_tmp;

					delete	p_frameRx;
				}
			}
		}
	}

	delete	p_frameTx;

	return STATUS_SUCCESS;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago               Data di creazione : 20/6/2002
 * Codice   :
 *
 * Note:  Richiede tutte le misure alla PCFOM con il frame PCCFOM_CMD_GET_
 *
 *
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */


NTSTATUS	NTPCFOMDevice::SetFOMMode(USHORT	w_FOMMode)
{
	NTSTATUS 		status;
	PCCIO_FR_STR*   p_frameTx = NULL;
	PCCIO_FR_STR*   p_frameRx = NULL;
	USHORT          w_tmp;

// Creo un frame per richiedere la configurazione dei trace.
	p_frameTx =	new	PCCIO_FR_STR(PCCFOM_CMD_SET_FOM_MODE,w_FOMMode,0);

// Se non c'e' memoria esco.
	if(p_frameTx	==	NULL)
		return STATUS_UNSUCCESSFUL;

// Trasmetto alla scheda PccFom il frame.
// Aspetto che arrivi il frame di risposta.
// Ricevo il frame.
// Esco se c'e' errore di comunicazione.

	status	=	DeviceTxFrame(p_frameTx);

	delete	p_frameTx;

	return status;
}
